@@ -22,13 +22,16 @@ import os
 import re
 import socket
 from binascii import unhexlify
+import random
 
 from pyasn1.codec.ber import encoder, decoder
 from pyasn1.error import SubstrateUnderrunError
 from pyasn1.type.univ import noValue
 
 from impacket import LOG
-from impacket.ldap.ldapasn1 import *
+from impacket.ldap.ldapasn1 import Filter, Control, SimplePagedResultsControl, ResultCode, Scope, DerefAliases, Operation, \
+    KNOWN_CONTROLS, CONTROL_PAGEDRESULTS, NOTIFICATION_DISCONNECT, KNOWN_NOTIFICATIONS, BindRequest, SearchRequest, \
+    SearchResultDone, LDAPMessage
 from impacket.ntlm import getNTLMSSPType1, getNTLMSSPType3
 from impacket.spnego import SPNEGO_NegTokenInit, TypesMech
 
@@ -76,7 +79,6 @@ class LDAPConnection:
         self._dstHost = 0
         self._socket = None
         self._baseDN = baseDN
-        self._messageId = 1
         self._dstIp = dstIp
 
         if url.startswith('ldap://'):
@@ -167,7 +169,7 @@ class LDAPConnection:
             else:
                 # retrieve domain information from CCache file if needed
                 if domain == '':
-                    domain = ccache.principal.realm['data']
+                    domain = ccache.principal.realm['data'].decode('utf-8')
                     LOG.debug('Domain retrieved from CCache: %s' % domain)
 
                 LOG.debug('Using Kerberos Cache: %s' % os.getenv('KRB5CCNAME'))
@@ -188,10 +190,10 @@ class LDAPConnection:
 
                 # retrieve user information from CCache file if needed
                 if user == '' and creds is not None:
-                    user = creds['client'].prettyPrint().split('@')[0]
+                    user = creds['client'].prettyPrint().split(b'@')[0].decode('utf-8')
                     LOG.debug('Username retrieved from CCache: %s' % user)
                 elif user == '' and len(ccache.principal.components) > 0:
-                    user = ccache.principal.components[0]['data']
+                    user = ccache.principal.components[0]['data'].decode('utf-8')
                     LOG.debug('Username retrieved from CCache: %s' % user)
 
         # First of all, we need to get a TGT for the user
@@ -322,15 +324,15 @@ class LDAPConnection:
 
             # NTLM Negotiate
             negotiate = getNTLMSSPType1('', domain)
-            bindRequest['authentication']['sicilyNegotiate'] = negotiate
+            bindRequest['authentication']['sicilyNegotiate'] = negotiate.getData()
             response = self.sendReceive(bindRequest)[0]['protocolOp']
 
             # NTLM Challenge
             type2 = response['bindResponse']['matchedDN']
 
             # NTLM Auth
-            type3, exportedSessionKey = getNTLMSSPType3(negotiate, str(type2), user, password, domain, lmhash, nthash)
-            bindRequest['authentication']['sicilyResponse'] = type3
+            type3, exportedSessionKey = getNTLMSSPType3(negotiate, bytes(type2), user, password, domain, lmhash, nthash)
+            bindRequest['authentication']['sicilyResponse'] = type3.getData()
             response = self.sendReceive(bindRequest)[0]['protocolOp']
         else:
             raise LDAPSessionError(errorString="Unknown authenticationChoice: '%s'" % authenticationChoice)
@@ -415,7 +417,7 @@ class LDAPConnection:
 
     def send(self, request, controls=None):
         message = LDAPMessage()
-        message['messageID'] = self._messageId
+        message['messageID'] = random.randrange(1, 2147483647)
         message['protocolOp'].setComponentByType(request.getTagSet(), request)
         if controls is not None:
             message['controls'].setComponents(*controls)
@@ -426,7 +428,7 @@ class LDAPConnection:
 
     def recv(self):
         REQUEST_SIZE = 8192
-        data = ''
+        data = b''
         done = False
         while not done:
             recvData = self._socket.recv(REQUEST_SIZE)
@@ -456,7 +458,6 @@ class LDAPConnection:
                 response.append(message)
             data = remaining
 
-        self._messageId += 1
         return response
 
     def sendReceive(self, request, controls=None):
@@ -465,9 +466,10 @@ class LDAPConnection:
 
     def _parseFilter(self, filterStr):
         try:
-            filterList = list(reversed(unicode(filterStr)))
-        except UnicodeDecodeError:
-            filterList = list(reversed(filterStr))
+            filterStr = filterStr.decode()
+        except AttributeError:
+            pass
+        filterList = list(reversed(filterStr))
         searchFilter = self._consumeCompositeFilter(filterList)
         if filterList:  # we have not consumed the whole filter string
             raise LDAPFilterSyntaxError("unexpected token: '%s'" % filterList[-1])
